classdef CDAQ < CNi9234Hardware
    %CDAQ Ni DAQ hardware object

    properties
        InputModules
        OutputModules
        DMADepth

        AI      % daq analog input object
        AO      % winsound analog output object
        Buffer
    end
    
    properties (SetObservable = true)
        nSamplesInBuffer    % number of samples in buffer
    end
    
    methods
        % Constructor
        function obj = CDAQ
            obj = obj@CNi9234Hardware();
            
            switch sysnbit
                case 32
                    a = daqhwinfo('nidaq');
                    obj.AI = eval(a.ObjectConstructorName{1});
                    info = daqhwinfo(obj.AI);
                    obj.DMADepth = 2e6;
                    switch info.DeviceName
                        case 'USB-9234'
                            obj.InputModules = CModule.NI9234b;
                        case 'USB-9215A (BNC)'
                            obj.InputModules = CModule.NI9215;
                    end
                    
                    try
                        a = daqhwinfo('winsound');
                        obj.AO = eval(a.ObjectConstructorName{strncmpi('analogoutput', a.ObjectConstructorName, length('analogoutput'))});
                        obj.OutputModules = CModule.WinSoundO; % TODO fix this
                    catch
                        obj.OutputModules = CModule.empty(0,1);
                        obj.AO = [];
                    end
                    
                case 64
                    try
                        d = daq.getDevices();
                        if ~isempty(d)
                            for i = 1 : length(d)
                                switch d(i).Model
                                    case 'USB-9234'
                                        obj.InputModules = CModule.NI9234b;
                                    case 'USB-9215A (BNC)'
                                        obj.InputModules = CModule.NI9215;
                                end
                            end
                            obj.DMADepth = 2e6;
                            obj.AI = daq.createSession('ni');
                        else
                            obj.AI = [];
                        end
                    catch
                        obj.AI = [];
                    end
            end
            if (isempty(obj.AI) || isempty(obj.AO))
                error('CDAQ:nohardware', 'No DAQ hardware found');
            end
        end
        
        function status = MonitorInit(obj, Channels)
            ChSel = cell2mat({Channels.isSelected});
            switch sysnbit
                case 32
                    addchannel(obj.AI, cell2mat({Channels(ChSel).PhysChNr})-1);
                case 64
                    obj.AI.addAnalogInputChannel('Dev1', cell2mat({Channels(ChSel).PhysChNr})-1, 'Voltage');
            end
            status = 0;
        end
        
        function [status, starttime] = MonitorStart(obj, Fs, nSamples)

            switch sysnbit
                case 32
                    obj.AI.SampleRate = Fs;
                    obj.AI.TriggerRepeat = Inf;
                    obj.AI.SamplesPerTrigger = nSamples;
                    start(obj.AI);
                case 64
                    obj.AI.Rate = Fs;
                    %                     obj.AI.NotifyWhenDataAvailableExceeds = nSamples;
                    obj.AI.IsContinuous = true;
                    obj.AI.addlistener('DataAvailable', @(src, data)queueData(obj, data));
                    obj.Buffer = zeros(0, length(obj.AI.Channels));
                    obj.AI.startBackground();
            end
            starttime = now;
            status = 0;
        end
        
        function queueData(obj, data)
            obj.Buffer = [
                obj.Buffer
                data.Data
                ];
        end
        
        function [status, voltage] = MonitorAcq(obj, nSamples)
            switch sysnbit
                case 32
                    voltage = getdata(obj.AI, nSamples);
                    obj.nSamplesInBuffer = obj.AI.SamplesAvailable*length(obj.AI.Channel);
                case 64
                    while size(obj.Buffer, 1) < nSamples
                        disp('waiting');
                        pause(.1);
                    end
                    disp('acquiring');
                    voltage = obj.Buffer(1:nSamples,:);
                    obj.Buffer = obj.Buffer(nSamples+1:end, :);
                    obj.nSamplesInBuffer = size(obj.Buffer,1);
            end
            status = 0;
        end
        
        function [status rest] = MonitorStop(obj)
            switch sysnbit
                case 32
                    stop(obj.AI);
                    if nargout > 0
                        nRes = obj.AI.SamplesAvailable;
                        if nRes > 0
                            rest = getdata(obj.AI, nRes);
                        else
                            rest = zeros(0, length(obj.AI.Channel));
                        end
                    else
                        flushdata(obj.AI);
                    end
                case 64
                    obj.AI.stop();
                    if nargout > 0
                        rest = obj.Buffer;
                    end
                    obj.Buffer = zeros(0, size(obj.Buffer,2));
            end
            status = 0;
        end
        
        function status = MonitorClose(obj)
            switch sysnbit
                case 32
                    delete(obj.AI.Channel);
                    flushdata(obj.AI);
                case 64
                    obj.AI.removeChannel(1:length(obj.AI.Channels));
                    obj.Buffer = [];
            end
            status = 0;
        end
        
        function status = TransferInit(obj, InputChannels, OutputChannels)
            ChSel = cell2mat({InputChannels.isSelected});
            addchannel(obj.AI, cell2mat({InputChannels(ChSel).PhysChNr})-1);
            ChSel = cell2mat({OutputChannels.isSelected});
            addchannel(obj.AO, cell2mat({OutputChannels(ChSel).PhysChNr}));
            status = 0;
        end
        
        function [status y]= TransferAcq(obj, nPre, nAverage, nPost, Fs, x)
            obj.AI.SampleRate = Fs;
            obj.AO.SampleRate = Fs;
            
            nTot = nPre + nAverage + nPost;
            nSamples = size(x,1);
            
            obj.AI.TriggerRepeat = 0;
            obj.AI.SamplesPerTrigger = nTot*nSamples;
            
            nCh = length(obj.AI.Channel);
            
            if size(x,2) == 1
                x = repmat(x, 1, nTot);
            end
            putdata(obj.AO, repmat(x(:), 1, length(obj.AO.Channel)));
            
            flushdata(obj.AI);
            
            start([obj.AI obj.AO]);
            
            voltage = getdata(obj.AI);
            voltage = reshape(voltage, nSamples, nTot, []);
            y = voltage(:,nPre+(1:nAverage), :);
            
            obj.nSamplesInBuffer = obj.AI.SamplesAvailable*nCh;
            
            stop(obj.AI);
            stop(obj.AO);
            
            flushdata(obj.AI);
            
            status = 0;
        end
        
        function status = TransferClose(obj)
            delete(obj.AI.Channel);
            delete(obj.AO.Channel);
            flushdata(obj.AI);
            status = 0;
        end
        
        function ImpactInit(obj, Channels)
        end;
        
        function ImpactArm(obj, nSamples, nPretrigSamples, Fs)
        end;
        
        function ImpactWaitForTrigger(obj, trigLevel, trigCh)
        end;
        
        function ImpactAcq(obj)
        end;
        
        function ImpactClose(obj)
        end;
    end
end
