classdef CRecord < CUiTab & CChSetupSensitive
    %CRECORD Nihu.Lab module for signal recording
    
    properties
        Hardware
        AcqSetup
        AcqSetupView
        LevelMeter
        LevelMeterView
        Streamer
        StreamerView
        FileSelect
        FileSelectView
        TimeElapsed
        TimeElapsedView
        
        isRunning
    end
    
    methods
        function obj = CRecord(ParentGroup, hardware, chsetup, acqsetup, levmeter)
            obj = obj@CUiTab(ParentGroup, 'RECORD_MODULE');
            obj = obj@CChSetupSensitive(chsetup);
            
            obj.Hardware = hardware;
            obj.AcqSetup = acqsetup;
            obj.LevelMeter = levmeter;
            
            obj.FileSelect = CFileSelect({'bin'});
            obj.TimeElapsed = CTimeElapsed();
            
            obj.Streamer = CStreamer;
            obj.InitFigure();
        end
        
        function RecordCallback(obj, hButton)
            %RECORDCALLBACK function called when start/stop recording button is pressed
            obj.isRunning = ~obj.isRunning;
            
            if ~obj.isRunning
                return;
            end
            
            obj.FileSelect.NextAvailable();
            obj.TimeElapsed.Reset();
            obj.Streamer.Initialize(fullfile(obj.FileSelect.Directory, obj.FileSelect.FileName), ...
                obj.ChannelSetup.InputChannels, obj.AcqSetup);
            
            set(hButton, 'String', CBabel.getInstance().getText('STOP_RECORDING'));
            
            % block interface
            set(gcbf, 'UserData', get(gcbf, 'CloseRequestFcn'));
            set(gcbf, 'CloseRequestFcn', '');
            obj.hParentGroup.block();
            obj.AcqSetupView.block();
            obj.FileSelectView.block();
            obj.StreamerView.unblock();
            obj.LevelMeterView.block();
            
            try
                [status, starttime] = obj.Hardware.MonitorStart(obj.AcqSetup.Fs, obj.AcqSetup.nSamples);
                if status < 0
                    error('NiHuLab:hardware', 'Hardware start error');
                end
                obj.Streamer.AddTime(starttime);
                
                while true
                    if ~obj.isRunning
                        break;
                    end
                    
                    [status, voltage] = obj.Hardware.MonitorAcq(obj.AcqSetup.nSamples);
                    if status < 0
                        error('NiHuLab:hardware', 'Hardware acquisition error');
                    end
                    obj.LevelMeter.UpdateLevels(voltage);
                    obj.TimeElapsed.Increase(obj.AcqSetup.T);
                    obj.Streamer.AddData(obj.ChannelSetup.voltage2eu(voltage));
                    
                    drawnow;
                end
                
                status = obj.Hardware.MonitorStop();
                if status < 0
                    error('NiHuLab:hardware', 'Hardware closing error');
                end
            catch ME
                errordlg(sprintf('%s\n%s', ME.identifier, ME.message), '!!! Error !!!', 'modal');
            end
            
            set(hButton, 'String', CBabel.getInstance().getText('START_RECORDING'));
            
            % unblock interface
            set(gcbf, 'CloseRequestFcn', get(gcbf, 'UserData'));
            obj.hParentGroup.unblock();
            obj.AcqSetupView.unblock();
            obj.FileSelectView.unblock();
            obj.LevelMeterView.unblock();
            obj.StreamerView.block();
            
            obj.Streamer.Finalize('verbose');
            
            obj.isRunning = false;
        end
    end
    
    methods
        function show(obj)
            %SHOW function called each time before the module is opened
            obj.isRunning = false;
            obj.LevelMeter.Reset();
            obj.TimeElapsed.Reset();
            obj.StreamerView.block();
            
            status = obj.Hardware.MonitorInit(obj.ChannelSetup.InputChannels);
        end
        
        function hide(obj)
            %SHOW function called each time before the module is closed
            
            status = obj.Hardware.MonitorClose();
        end
    end
    
    methods (Access = private)
        function InitFigure(obj)
            obj.AcqSetupView = CAcqSetupView(obj.hContainer, obj.AcqSetup);
            obj.LevelMeterView = CLevelMeterView(obj.hContainer, obj.LevelMeter, 100);
            obj.FileSelectView = CFileSelectView(obj.hContainer, obj.FileSelect);
            obj.TimeElapsedView = CTimeElapsedView(obj.hContainer, obj.TimeElapsed);
            obj.StreamerView = CStreamerView(obj.hContainer, obj.Streamer);
            h4 = CHardwareView(obj.hContainer, obj.Hardware);
            butAcq = uicontrol('parent', obj.hContainer,...
                'Style', 'PushButton', ...
                'Tag', 'START_RECORDING', ...
                'String', CBabel.getInstance().getText('START_RECORDING'), ...
                'Callback', @(src, event)RecordCallback(obj, src));
            w = get(obj.AcqSetupView.hPanel, 'position');
            w = w(3);
            Layout = {
                {obj.AcqSetupView.hPanel, [w 0]};
                {obj.FileSelectView.hPanel, [w 0]};
                {obj.StreamerView.hPanel, [w 0]};
                {obj.TimeElapsedView.hPanel, [w 0]};
                {butAcq, [w 0]};
                {obj.LevelMeterView.hPanel, [w -1]};
                {h4.hPanel, [w 0]}
                };
            GuiLayout.ApplyLayout(obj.hContainer, Layout);
            movegui(obj.hContainer, 'center');
            movegui(obj.hContainer, 'onscreen');
        end
    end
end
